
# q-expansion of newform 169.4.b.c, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 169
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2, [-1, 0]]]
aps_data = [[0, 4], [2, 0], [0, 17], [0, -20], [0, 32], [0, 0], [13, 0], [0, 30], [-78, 0], [197, 0], [0, -74], [0, 227], [0, -165], [156, 0], [0, 162], [93, 0], [0, 864], [145, 0], [0, 862], [0, 654], [0, -215], [-76, 0], [0, 628], [0, 266], [0, 238], [819, 0], [-1638, 0], [522, 0], [0, -1634], [327, 0], [2158, 0], [730, 0], [0, -1671], [912, 0], [0, -2115], [0, -514], [2901, 0], [0, -2360], [0, -280], [-1326, 0], [-4264, 0], [403, 0], [-1246, 0], [0, -267], [0, 1278], [-4238, 0], [3070, 0], [0, -5378], [0, -3974], [0, 6298], [-4030, 0], [0, -984], [0, -943], [2730, 0], [1885, 0], [4032, 0], [4006, 0], [0, 4296], [5551, 0], [0, 5557], [-3120, 0], [0, -8301], [0, -8678], [-8658, 0], [-5250, 0], [0, 6413], [0, 3488], [1833, 0], [7230, 0], [0, 5258], [0, 3163], [0, 10068], [7438, 0], [-9683, 0], [0, -1062], [0, -3532], [11063, 0], [0, 5986], [0, -5935], [0, -15089], [-10814, 0], [0, -6535], [0, 1980], [6929, 0], [4576, 0], [-8812, 0], [0, -1918], [0, -11761], [0, 901], [0, -1372], [6396, 0], [0, -3270], [0, 19920], [-6552, 0], [0, 1746], [14692, 0], [0, 8077], [11247, 0], [2732, 0], [0, 18375], [-10346, 0], [0, -345], [8580, 0], [19682, 0], [-26624, 0], [0, -14101], [0, 1408], [0, 1241], [11078, 0], [-13817, 0], [8270, 0], [0, 22273], [0, -18989], [0, -72], [0, 23380], [-6383, 0], [0, -17104], [-6994, 0], [-5250, 0], [-4340, 0], [0, 4179], [-22867, 0], [5410, 0], [0, 13578], [0, 12744], [-16406, 0], [0, -709], [7644, 0], [15808, 0], [0, -2583], [0, -4076], [0, -34056], [364, 0], [-6914, 0], [0, -13982], [0, -18066], [0, 14434], [0, 15398], [36842, 0], [41511, 0], [0, 23066], [0, 28838], [-27456, 0], [0, 33572], [45799, 0], [0, -32286], [0, -20937], [7189, 0], [-32498, 0], [0, 8428], [0, -6847], [-29731, 0], [23738, 0], [27588, 0], [37128, 0], [20516, 0], [-21006, 0], [0, 20427], [33191, 0], [0, -36422], [0, 39630], [-57642, 0], [0, 2162], [-19758, 0], [0, -12489], [0, 28658], [-42794, 0], [-52583, 0]]
